package com.upyuns.platform.rs.datacenter.utils.gdal;

import com.github.wxiaoqi.security.common.exception.BaseException;
import com.github.wxiaoqi.security.common.util.process.ResultCode;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGEncodeParam;
import com.sun.image.codec.jpeg.JPEGImageEncoder;
import com.upyuns.platform.rs.datacenter.vo.ImageStorageVo;
import org.gdal.gdal.Dataset;
import org.gdal.gdal.Driver;
import org.gdal.gdal.gdal;
import org.gdal.gdalconst.gdalconstConstants;
import org.gdal.osr.CoordinateTransformation;
import org.gdal.osr.SpatialReference;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;

public class ImageOperateTools {

    /**
     * tif 文件转 为图片
     * @param inputPath
     * @param outputPath
     * @return
     */
    public static Boolean tif2Pic(String inputPath, String outputPath) {
        //注册GDAL
        gdal.AllRegister();
        //设置中文
        gdal.SetConfigOption("gdal_FILENAME_IS_UTF8", "YES");

        //只读方式读取数据
        Dataset sourceData = gdal.Open(inputPath, gdalconstConstants.GA_ReadOnly);
        if (sourceData == null) {
            return false;
        }
        Driver hDriver = sourceData.GetDriver();
        hDriver.CreateCopy(outputPath, sourceData);

        hDriver.delete();
        sourceData.delete();

        return true;
    }

    /**
     * 获取影像四点坐标
     * @param inputPath
     * @return
     */
    public static ImageStorageVo getWGS84BoundingBox(String inputPath, ImageStorageVo imageStorageVo)throws BaseException {
        //注册GDAL
        gdal.AllRegister();
        //设置中文
        gdal.SetConfigOption("gdal_FILENAME_IS_UTF8", "YES");

        //只读方式读取数据
        Dataset sourceData = gdal.Open(inputPath, gdalconstConstants.GA_ReadOnly);
        if (sourceData == null) {
            throw new BaseException("文件错误", ResultCode.FAILED_CODE);
        }

        //获取源图像crs     获取源影像的坐标参考
        String srcWkt = sourceData.GetProjectionRef();
        //构造投影坐标系统的空间参考(srcWkt)
        SpatialReference srcCrs = new SpatialReference(srcWkt);
        sourceData.delete();
        //设置输出图像的坐标
        SpatialReference oLatLong;
        //获取该投影坐标系统中的地理坐标系
        oLatLong = srcCrs.CloneGeogCS();
        //构造一个从投影坐标系到地理坐标系的转换关系
        CoordinateTransformation ct = new CoordinateTransformation(srcCrs, oLatLong);

        int xSize = sourceData.GetRasterXSize(); // 栅格尺寸
        int ySize = sourceData.GetRasterYSize(); //
        double[] geoTransform = sourceData.GetGeoTransform();
        // 图像范围
        // 东西方向空间分辨率  --->像素宽度
        double wSrc = geoTransform[1];
        // 南北方向空间分辨率 ---> 像素高度
        double hSrc = geoTransform[5];
        // x方向旋转角
        double xRotate = geoTransform[2];
        // y方向旋转角
        double yRotate = geoTransform[4];

        // 左上角x坐标,y坐标 ---> 影像 左上角 投影坐标
        double xMin = geoTransform[0];
        double yMax = geoTransform[3];

        // 右下角 x坐标,y坐标
        double xMax = geoTransform[0] + xSize * wSrc + ySize * xRotate;
        double yMin = geoTransform[3] + xSize * yRotate + ySize * hSrc;

        double a[] = ct.TransformPoint(xMin, yMax);
        double b[] = ct.TransformPoint(xMax, yMin);
        if (imageStorageVo == null) {
            imageStorageVo = new ImageStorageVo();
        }
        //左上角坐标
        imageStorageVo.setLeftTopLon(new BigDecimal(xMin + ""));
        imageStorageVo.setLeftTopLat(new BigDecimal(yMax + ""));
        //左下角坐标
        imageStorageVo.setLeftBottomLon(new BigDecimal(xMin + ""));
        imageStorageVo.setLeftBottomLat(new BigDecimal(yMin + ""));
        //右下角坐标
        imageStorageVo.setRightBottomLon(new BigDecimal(xMax + ""));
        imageStorageVo.setRightBottomLat(new BigDecimal(yMin + ""));
        //右上角坐标
        imageStorageVo.setRightBottomLon(new BigDecimal(xMax + ""));
        imageStorageVo.setRightBottomLat(new BigDecimal(yMax + ""));
        BigDecimal two = new BigDecimal("2");
        BigDecimal centerPointLon = imageStorageVo.getLeftTopLon().add(imageStorageVo.getRightBottomLon().divide(two));
        BigDecimal centerPointLat = imageStorageVo.getLeftTopLat().add(imageStorageVo.getRightBottomLat().divide(two));
        imageStorageVo.setImageCenterPointLon(centerPointLon);
        imageStorageVo.setImageCenterPointLat(centerPointLat);
        return imageStorageVo;
    }

    //jpeg图片压缩处理
    public static Boolean zipImageFile(String oldFile, String newFile, int width, float quality) {
        if (oldFile == null) {
            return false;
        }
        /**对服务器上的临时文件进行处理 */
        ImageIcon imageIcon = new ImageIcon(oldFile);
        int iconWidth = imageIcon.getIconWidth();
        int iconHeight = imageIcon.getIconHeight();
        int height = iconHeight * width / iconWidth;
        return zipImageFile(oldFile, newFile, width, height, quality);
    }

    //jpeg图片压缩处理
    public static Boolean zipImageFile(String oldFile, String newFile, int width, int height, float quality) {
        if (oldFile == null) {
            return null;
        }
        try {
            /**对服务器上的临时文件进行处理 */
            Image srcFile = ImageIO.read(new File(oldFile));
            /** 宽,高设定 */
            BufferedImage tag = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
            tag.getGraphics().drawImage(srcFile, 0, 0, width, height, null);

            /** 压缩之后临时存放位置 */
            FileOutputStream out = new FileOutputStream(newFile);
            JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
            JPEGEncodeParam jep = JPEGCodec.getDefaultJPEGEncodeParam(tag);
            /** 压缩质量 */
            jep.setQuality(quality, true);
            encoder.encode(tag, jep);
            out.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            return false;
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }
}
